import unittest

import visgraph.graphcore as v_graphcore

s1paths = [
    ('a','c','f'),
    ('a','b','d','f'),
    ('a','b','e','f'),
]
s2paths = [
    ('a','b'),
    ('a','b','c'),
]

misc_graph_state1_nodes = [
    ('a', {'rootnode': True}),
    ('b', {'bar': [1, 2, 3, 4], 'baz': {1: 'blah'}, 'foo': 1234}),
    ('c', {}),
    ('d', {}),
    ('e', {}),
    ('f', {}),
    ('test1', {'bar': [1, 2, 3, 4], 'baz': {1: 'blah'}, 'foo': 1234})
]

misc_graph_state1_edges = [
    ('801b27a16048ab6a117aa8181a70da45', 'a', 'b', {}),
    ('f67a14a063e19e667295479881f74655', 'a', 'c', {}),
    ('3da9da188034739bd13c3c5ebc741028', 'c', 'f', {}),
    ('1cf9d6655169edb8d9eef8bd102a537b', 'b', 'd', {}),
    ('e76d0c84bc8233f6b7e38bafae46495f', 'b', 'e', {}),
    ('2bba716151cdd42e91dcfdaf45c8539a', 'd', 'f', {}),
    ('239039cc7bfa4de28f816519e70ded53', 'e', 'f', {}),
    ('90a3ba421614e4c7f0a6ddde5bbc8e2c',
      'b',
      'test1',
      {'bar': [1, 2, 3, 4],
       'baz': {1: 'blah'},
       'blad': 1234,
       'bled': {1: 'blah'},
       'blod': [1, 2, 3, 4],
       'foo': 1234}),
    ('f75197278c15a24132cf35cc92540b3d',
      'test1',
      'c',
      {'bar': [1, 2, 3, 4], 'baz': {1: 'blah'}, 'foo': 1234})
]


misc_graph_state2_nodes = [
    ('a', {'rootnode': True}),
    ('b', {'baz': {1: 'blah'}}),
    ('c', {}),
    ('d', {}),
    ('e', {}),
    ('f', {}),
    ('test1', {'bar': [1, 2, 3, 4], 'baz': {1: 'blah'}, 'foo': 1234})
]

misc_graph_state2_edges = [
    ('801b27a16048ab6a117aa8181a70da45', 'a', 'b', {}),
    ('f67a14a063e19e667295479881f74655', 'a', 'c', {}),
    ('3da9da188034739bd13c3c5ebc741028', 'c', 'f', {}),
    ('1cf9d6655169edb8d9eef8bd102a537b', 'b', 'd', {}),
    ('e76d0c84bc8233f6b7e38bafae46495f', 'b', 'e', {}),
    ('2bba716151cdd42e91dcfdaf45c8539a', 'd', 'f', {}),
    ('239039cc7bfa4de28f816519e70ded53', 'e', 'f', {}),
    ('90a3ba421614e4c7f0a6ddde5bbc8e2c',
      'b',
      'test1',
      {'bar': [1, 2, 3, 4],
       'baz': {1: 'blah'},
       'blad': 1234,
       'bled': {1: 'blah'},
       'blod': [1, 2, 3, 4],
       'foo': 1234}),
    ('f75197278c15a24132cf35cc92540b3d', 'test1', 'c', {'bar': [1, 2, 3, 4]})
]


misc_graph_state3_nodes = [
    ('a', {'rootnode': True}),
    ('b', {'baz': {1: 'blah'}}),
    ('c', {}),
    ('d', {}),
    ('e', {}),
    ('f', {})
]

misc_graph_state3_edges = [
    ('801b27a16048ab6a117aa8181a70da45', 'a', 'b', {}),
    ('f67a14a063e19e667295479881f74655', 'a', 'c', {}),
    ('3da9da188034739bd13c3c5ebc741028', 'c', 'f', {}),
    ('1cf9d6655169edb8d9eef8bd102a537b', 'b', 'd', {}),
    ('e76d0c84bc8233f6b7e38bafae46495f', 'b', 'e', {}),
    ('2bba716151cdd42e91dcfdaf45c8539a', 'd', 'f', {}),
    ('239039cc7bfa4de28f816519e70ded53', 'e', 'f', {})
]


class GraphCoreTest(unittest.TestCase):

    def getSampleGraph1(self):
        # simple branching/merging graph
        g = v_graphcore.HierGraph()

        g.addHierRootNode('a')
        for c in ('b','c','d','e','f'):
            g.addNode(c)

        g.addEdgeByNids('a','b')
        g.addEdgeByNids('a','c')
        g.addEdgeByNids('c','f')
        g.addEdgeByNids('b','d')
        g.addEdgeByNids('b','e')
        g.addEdgeByNids('d','f')
        g.addEdgeByNids('e','f')

        return g

    def getSampleGraph2(self):
        # primitive loop graph
        g = v_graphcore.HierGraph()

        g.addHierRootNode('a')
        for c in ('b','c'):
            g.addNode(c)

        g.addEdgeByNids('a','b')
        g.addEdgeByNids('b','b')
        g.addEdgeByNids('b','c')

        return g

    def getSampleGraph3(self):
        # flat loop graph
        g = v_graphcore.HierGraph()

        g.addHierRootNode('a')
        for c in ('b','c','d'):
            g.addNode(c)

        g.addEdgeByNids('a','b')
        g.addEdgeByNids('b','c')
        g.addEdgeByNids('c','b')
        g.addEdgeByNids('c','d')

        return g

    def test_visgraph_pathscount(self):
        g = self.getSampleGraph1()
        self.assertEqual(g.getHierPathCount(), 3)

        g = self.getSampleGraph2()
        self.assertEqual(g.getHierPathCount(), 1)

        g = self.getSampleGraph3()
        self.assertEqual(g.getHierPathCount(), 1)

    def assertPathsFrom(self, g, paths):
        allpaths = set(paths)
        root = g.getNode('a')
        for path in g.getHierPathsFrom(root):
            nids = tuple([ n[0] for (n,e) in path])
            self.assertIn(nids,allpaths)
            allpaths.remove(nids)
        self.assertFalse(allpaths)

    def test_visgraph_pathsfrom(self):
        self.assertPathsFrom( self.getSampleGraph1(), s1paths)
        self.assertPathsFrom( self.getSampleGraph2(), s2paths)

    def assertPathsTo(self, g, nid, paths):
        allpaths = set(paths)
        node = g.getNode(nid)
        for path in g.getHierPathsTo(node):
            nids = tuple([ n[0] for (n,e) in path])
            self.assertIn(nids,allpaths)
            allpaths.remove(nids)
        self.assertFalse(allpaths)

    def test_visgraph_pathsto(self):
        '''
        '''
        self.assertPathsTo( self.getSampleGraph1(), 'f', s1paths)
        self.assertPathsTo( self.getSampleGraph2(), 'c', [ ('a','b','c'), ])

    def assertPathsThru(self, g, nid, paths):
        allpaths = set(paths)
        node = g.getNode(nid)
        for path in g.getHierPathsThru(node):
            nids = tuple([ n[0] for (n,e) in path])
            self.assertIn(nids,allpaths)
            allpaths.remove(nids)
        self.assertFalse(allpaths)

    def test_visgraph_paththru(self):
        self.assertPathsThru( self.getSampleGraph1(),'b',[('a','b','d','f'),('a','b','e','f')])
        self.assertPathsThru( self.getSampleGraph2(),'b',[('a','b'),('a','b','c'),])

    def test_visgraph_nodeprops(self):
        g = v_graphcore.Graph()
        a = g.addNode('a')

        g.setNodeProp(a,'foo','bar')

        self.assertEqual(a[1].get('foo'), 'bar')

        self.assertTrue( a in g.getNodesByProp('foo') )
        self.assertTrue( a in g.getNodesByProp('foo','bar') )
        self.assertFalse( a in g.getNodesByProp('foo','blah') )

        g.delNodeProp(a,'foo')

        self.assertFalse( a in g.getNodesByProp('foo') )
        self.assertFalse( a in g.getNodesByProp('foo','bar') )
        self.assertIsNone(a[1].get('foo'))

    def test_visgraph_edgeprops(self):
        g = v_graphcore.Graph()
        a = g.addNode('a')
        b = g.addNode('b')

        e = g.addEdge(a,b)
        g.setEdgeProp(e,'foo','bar')

        self.assertEqual(e[3].get('foo'),'bar')

        self.assertTrue( e in g.getEdgesByProp('foo') )
        self.assertTrue( e in g.getEdgesByProp('foo','bar') )
        self.assertFalse( e in g.getEdgesByProp('foo','blah') )

        g.delEdgeProp(e,'foo')
        self.assertFalse( e in g.getEdgesByProp('foo') )
        self.assertFalse( e in g.getEdgesByProp('foo','bar') )
        self.assertIsNone(e[3].get('foo'))

    def test_visgraph_subcluster(self):

        g = v_graphcore.Graph()

        a = g.addNode('a')
        b = g.addNode('b')
        c = g.addNode('c')

        d = g.addNode('d')
        e = g.addNode('e')
        r = g.addNode('f')

        g.addEdgeByNids('a','b')
        g.addEdgeByNids('a','c')

        g.addEdgeByNids('d','e')
        g.addEdgeByNids('d','f')

        subs = g.getClusterGraphs()

        self.assertEqual(len(subs),2)

        subtests = [ set(['a','b','c']), set(['d','e','f']) ]

        for sub in subs:
            if sub.getNode('a'):
                self.assertIsNone(sub.getNode('d'))
                self.assertIsNone(sub.getNode('e'))
                self.assertIsNone(sub.getNode('f'))

                akids = [ edge[2] for edge in sub.getRefsFromByNid('a') ]

                self.assertTrue('b' in akids )
                self.assertTrue('c' in akids )

            elif sub.getNode('d'):
                self.assertIsNone(sub.getNode('a'))
                self.assertIsNone(sub.getNode('b'))
                self.assertIsNone(sub.getNode('c'))

                dkids = [ edge[2] for edge in sub.getRefsFromByNid('d') ]

                self.assertTrue('e' in dkids )
                self.assertTrue('f' in dkids )

            else:
                raise Exception('Invalid SubCluster!')


    def test_visgraph_formnode(self):
        g = v_graphcore.Graph()

        def wootctor(n):
            g.setNodeProp(n,'lul',1)

        n1 = g.formNode('woot', 10, ctor=wootctor)
        self.assertEqual( n1[1].get('lul'), 1 )

        g.setNodeProp(n1, 'lul', 2)
        g.setNodeProp(n1, 'foo', 'bar')

        n2 = g.formNode('woot', 20, ctor=wootctor)
        n3 = g.formNode('woot', 10, ctor=wootctor)

        self.assertEqual( n1[0], n3[0] )
        self.assertEqual( n1[1].get('lul'), 2)
        self.assertEqual( n3[1].get('foo'), 'bar')
        self.assertNotEqual( n1[0], n2[0])

    def test_visgraph_add_del_misc_properties(self):
        g = self.getSampleGraph1()
        na = g.getNode('a')
        nb = g.getNode('b')

        # Test Adding Node with good and bad properties
        ntest1 = g.addNode('test1', foo=1234, bar=[1, 2, 3, 4], baz={1: 'blah'})

        # Test Adding good and bad properties to an existing node
        g.setNodeProp(nb, 'foo', 1234)
        g.setNodeProp(nb, 'bar', [1, 2, 3, 4])
        g.setNodeProp(nb, 'baz', {1: 'blah'})

        ## test reverse lookups.  lists and dicts aren't stored because 
        ##      they don't store well as dictionary keys
        self.assertEqual(list(g.nodeprops['foo'].keys()), [1234])
        self.assertEqual(list(g.nodeprops['bar'].keys()), [])
        self.assertEqual(list(g.nodeprops['baz'].keys()), [])

        # Test Adding Edge with good and bad properties
        etest1 = g.addEdge(nb, ntest1, foo=1234, bar=[1, 2, 3, 4], baz={1: 'blah'})
        etest2 = g.addEdgeByNids('test1', 'c', foo=1234, bar=[1, 2, 3, 4], baz={1: 'blah'})

        # Test Adding good and bad properties to an existing edge
        g.setEdgeProp(etest1, 'blad', 1234)
        g.setEdgeProp(etest1, 'blod', [1, 2, 3, 4])
        g.setEdgeProp(etest1, 'bled', {1: 'blah'})

        ## test reverse lookups.  lists and dicts aren't stored because 
        ##      they don't store well as dictionary keys
        self.assertEqual(list(g.edgeprops['blad'].keys()), [1234])
        self.assertEqual(list(g.edgeprops['blod'].keys()), [])
        self.assertEqual(list(g.edgeprops['bled'].keys()), [])

        ## Validate State of Graph
        self.assertEqual(g.getNodes(), misc_graph_state1_nodes)
        test_edges = [(nid1, nid2, eprops) for eid, nid1, nid2, eprops in g.getEdges()]
        ctrl_edges = [(nid1, nid2, eprops) for eid, nid1, nid2, eprops in misc_graph_state1_edges]
        self.assertEqual(test_edges, ctrl_edges)

        # Test Deleting good and bad properties from Node and Edge
        g.delNodeProp(nb, 'foo')
        g.delNodeProp(nb, 'bar')
        g.delEdgeProp(etest2, 'foo')
        g.delEdgeProp(etest2, 'baz')

        ## Validate State of Graph
        self.assertEqual(g.getNodes(), misc_graph_state2_nodes)
        test_edges = [(nid1, nid2, eprops) for eid, nid1, nid2, eprops in g.getEdges()]
        ctrl_edges = [(nid1, nid2, eprops) for eid, nid1, nid2, eprops in misc_graph_state2_edges]
        self.assertEqual(test_edges, ctrl_edges)



        # Test Deleting nodes and edges
        g.delEdge(etest2)
        g.delNode(ntest1)

        ## Validate State of Graph
        self.assertEqual(g.getNodes(), misc_graph_state3_nodes)
        test_edges = [(nid1, nid2, eprops) for eid, nid1, nid2, eprops in g.getEdges()]
        ctrl_edges = [(nid1, nid2, eprops) for eid, nid1, nid2, eprops in misc_graph_state3_edges]
        self.assertEqual(test_edges, ctrl_edges)




import os
import sys
import binascii

import vstruct
from vstruct.primitives import *

#HEAD_TYPE_MARKER        = 0x72          #marker block
#HEAD_TYPE_ARCHIVE       = 0x73          #archive header
#HEAD_TYPE_FILE_HDR      = 0x74          #file header
#HEAD_TYPE_OLD_COMMENT   = 0x75          #old style comment header
#HEAD_TYPE_OLD_AUTH      = 0x76          #old style authenticity information
#HEAD_TYPE_OLD_SUBBLOCK  = 0x77          #old style subblock
#HEAD_TYPE_OLD_RECOVERY  = 0x78          #old style recovery record
#HEAD_TYPE_OLD_AUTH2     = 0x79          #old style authenticity information
#HEAD_TYPE_SUBBLOCK      = 0x7a          #subblock

SFX_MODMAX  = 1024000 # one meg
RAR4_SIGNATURE = binascii.unhexlify('526172211a0700')
RAR5_SIGNATURE = binascii.unhexlify('526172211a070100')

def getRarOffset(fd):
    cur = fd.tell()
    head = fd.read(SFX_MODMAX * 2)
    fd.seek(cur)

    offset = head.find(RAR5_SIGNATURE)
    if offset != -1:
        return ( (5,0,0), offset + 8 )

    offset = head.find(RAR4_SIGNATURE)
    if offset != -1:
        return ( (4,0,0), offset + 7 )

    return None

# Header Types
MARK_HEAD       = 0x72
MAIN_HEAD       = 0x73
FILE_HEAD       = 0x74
COMM_HEAD       = 0x75
AV_HEAD         = 0x76
SUB_HEAD        = 0x77
PROTECT_HEAD    = 0x78
SIGN_HEAD       = 0x79
NEWSUB_HEAD     = 0x7a
ENDARC_HEAD     = 0x7b

# Main Header Flags
MHD_VOLUME          = 0x0001
MHD_COMMENT         = 0x0002
MHD_LOCK            = 0x0004
MHD_SOLID           = 0x0008
MHD_PACK_COMMENT    = 0x0010
MHD_AV              = 0x0020
MHD_PROTECT         = 0x0040
MHD_PASSWORD        = 0x0080    # The archive is password encrypted
MHD_FIRSTVOLUME     = 0x0100
MHD_ENCRYPTVER      = 0x0200

LHD_SPLIT_BEFORE   = 0x0001
LHD_SPLIT_AFTER    = 0x0002
LHD_PASSWORD       = 0x0004
LHD_COMMENT        = 0x0008
LHD_SOLID          = 0x0010
LHD_WINDOWMASK     = 0x00e0
LHD_WINDOW64       = 0x0000
LHD_WINDOW128      = 0x0020
LHD_WINDOW256      = 0x0040
LHD_WINDOW512      = 0x0060
LHD_WINDOW1024     = 0x0080
LHD_WINDOW2048     = 0x00a0
LHD_WINDOW4096     = 0x00c0
LHD_DIRECTORY      = 0x00e0
LHD_LARGE          = 0x0100
LHD_UNICODE        = 0x0200
LHD_SALT           = 0x0400
LHD_VERSION        = 0x0800
LHD_EXTTIME        = 0x1000
LHD_EXTFLAGS       = 0x2000

SKIP_IF_UNKNOWN    = 0x4000
LONG_BLOCK         = 0x8000

SIZE_SALT30        = 8
SIZE_SALT50        = 16
SIZE_IV            = 16

CRYPT_NONE         = 0
CRYPT_RAR13        = 1
CRYPT_RAR15        = 2
CRYPT_RAR20        = 3
CRYPT_RAR30        = 4
CRYPT_RAR50        = 5

CRYPT_BLOCKSIZE    = 16

class RarChunkUnkn(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.CHUNK_BYTES = v_bytes()

class MainHeader(vstruct.VStruct):
    def __init__(self):
        vstruct.VStruct.__init__(self)
        self.HighPosAV  = v_uint16()
        self.PosAV      = v_uint32()
        self.EncryptVer = v_uint8()

#class Rar4BaseBlock

class Rar4Block(vstruct.VStruct):

    def __init__(self):
        vstruct.VStruct.__init__(self)

        self.HEAD_CRC       = v_uint16()
        self.HEAD_TYPE      = v_uint8()
        self.HEAD_FLAGS     = v_uint16()
        self.HEAD_SIZE      = v_uint16()
        self.HEAD_DATA      = vstruct.VStruct()

class MAIN_HEADER(Rar4Block):
    def __init__(self):
        Rar4Block.__init__(self)
        self.HEAD_DATA.HighPosAv    = v_uint16()
        self.HEAD_DATA.PosAV        = v_uint32()
        if self.HEAD_FLAGS & MHD_ENCRYPTVER:
            self.HEAD_DATA.EncryptVer   = v_uint8()

class FILE_HEADER(Rar4Block):
    def __init__(self):
        Rar4Block.__init__(self)
        self.HEAD_DATA.PackSize     = v_uint32()
        self.HEAD_DATA.UnpSize      = v_uint32()
        self.HEAD_DATA.HostOs       = v_uint8()
        self.HEAD_DATA.FileCrc      = v_uint32()
        self.HEAD_DATA.FileTime     = v_uint32()
        self.HEAD_DATA.UnpVer       = v_uint8()
        self.HEAD_DATA.Method       = v_uint8()
        self.HEAD_DATA.NameSize     = v_uint16()
        self.HEAD_DATA.FileAttr     = v_uint32()

        if self.HEAD_FLAGS & LHD_LARGE:
            self.HEAD_DATA.HighPackSize = v_uint32()
            self.HEAD_DATA.HighUnpSize  = v_uint32()

        filename = v_str()
        self.HEAD_DATA.FileName     = filename

        if self.HEAD_FLAGS & LHD_SALT:
            self.HEAD_DATA.Salt     = v_bytes(size=8)

        if self.HEAD_FLAGS & LHD_EXTTIME:
            raise Exception("FIXME supprort LHD_EXTTIME")

        def setFileNameSize(x):
            filename.vsSetLength( self.HEAD_DATA.NameSize )

        self.HEAD_DATA.vsAddParseCallback('NameSize',setFileNameSize)

        #self.HEAD_PAD       = vstruct.VStruct()

        #self.ADD_SIZE       = v_uint32()
        #self.BLOCK_DATA     = vstruct.VStruct()
        #self._known_block   = False

    #def pcb_HEAD_DATA(self):
        #remain = len(self) % 16
        #if remain:
            #self.HEAD_PAD.pad = v_bytes(size=16-remain)

    #def pcb_HEAD_SIZE(self):

        #if self.HEAD_TYPE == MAIN_HEAD:
            #self.HEAD_DATA.HighPosAv    = v_uint16()
            #self.HEAD_DATA.PosAV        = v_uint32()
            #if self.HEAD_FLAGS & MHD_ENCRYPTVER:
                #self.HEAD_DATA.EncryptVer   = v_uint8()
            #return

        #if self.HEAD_TYPE == FILE_HEAD:
            #self.HEAD_DATA.PackSize     = v_uint32()
            #self.HEAD_DATA.UnpSize      = v_uint32()
            #self.HEAD_DATA.HostOs       = v_uint8()
            #self.HEAD_DATA.FileCrc      = v_uint32()
            #self.HEAD_DATA.FileTime     = v_uint32()
            #self.HEAD_DATA.UnpVer       = v_uint8()
            #self.HEAD_DATA.Method       = v_uint8()
            #self.HEAD_DATA.NameSize     = v_uint16()
            #self.HEAD_DATA.FileAttr     = v_uint32()

            #if self.HEAD_FLAGS & LHD_LARGE:
                #self.HEAD_DATA.HighPackSize = v_uint32()
                #self.HEAD_DATA.HighUnpSize  = v_uint32()

            #filename = v_str()
            #self.HEAD_DATA.FileName     = filename

            #if self.HEAD_FLAGS & LHD_SALT:
                #self.HEAD_DATA.Salt     = v_bytes(size=8)

            #if self.HEAD_FLAGS & LHD_EXTTIME:
                #raise Exception("FIXME supprort LHD_EXTTIME")

            #def setFileNameSize(x):
                #filename.vsSetLength( self.HEAD_DATA.NameSize )

            #self.HEAD_DATA.vsAddParseCallback('NameSize',setFileNameSize)
            #return

            #self.HEAD_DATA.NameSize     = v_uint32()
            #self.HEAD_DATA.NameSize     = v_uint32()

            #if not self.HEAD_FLAGS & MHD_ENCRYPTVER:
                #self.BLOCK_DATA.EncryptVer = vstruct.VStruct()

    #def pcb_HEAD_FLAGS(self):
        ## a proto callback for the header
        #if self.HEAD_FLAGS & LONG_BLOCK:
            #self.ADD_SIZE = v_uint32()
        #else:
            #self.ADD_SIZE = vstruct.VStruct()

        #if self.HEAD_TYPE == MAIN_HEAD and self.HEAD_FLAGS & MHD_PASSWORD:
                #self.BLOCK_DATA.Salt = v_bytes(size=8)

    #def pcb_ADD_SIZE(self):

        # first things first, needs salt?
        #if self.HEAD_FLAGS & MHD_PASSWORD:
            #self.BLOCK_DATA.Salt = v_bytes(size=8)

        #hsize = 7
        #totsize = self.HEAD_SIZE
#
        #if not isinstance(self.ADD_SIZE, vstruct.VStruct):
            #hsize += 4
            #totsize += self.ADD_SIZE

        # We will *now* use TYPE to find out our chunk guts
        #if not self._known_block:
            #self.BLOCK_DATA = v_bytes(totsize - hsize)

import hashlib
rounds = 0x40000
roundsdiv = rounds / 16
#iblist = [ struct.pack('<I',i)[:3] for i in range(rounds) ]

def initIvKey30(passwd,salt):
    aesiv = [None] * 16
    aeskey = [None] * 16

    passb = passwd.encode('utf-16le')
    initkey = passb + salt

    sha1hash = hashlib.sha1()
    #sha1hash = rarsha()
    # crazy russian awesomeness/paranoia
    for i in range(rounds): # srsly?!?! fscking russians ;)
        sha1hash.update(initkey)
        ib = struct.pack('<I',i)
        sha1hash.update( ib[:3] )
        #sha1hash.update( iblist[i] )

        if i % roundsdiv == 0:
            digest = sha1hash.digest()
            #digest = sha1hash.done()
            aesiv[ i / roundsdiv ] = digest[-1]

    endswap = struct.unpack_from('<4I', sha1hash.digest())
    aeskey  = struct.pack('>4I', *endswap)
    #digest = sha1hash.digest()
    #for i in range(4):
        #for j in range(4):
            #aeskey[ (i*4) + j ] = chr( (digest[i] >> (j*8)) & 0xff )

    return ''.join(aesiv), aeskey

def aesInit(iv,key):
    from Crypto.Cipher import AES
    return AES.new(key, AES.MODE_CBC, iv)

class NoRarFd(Exception):pass
class MissingRarSig(Exception):pass
class PasswordRequired(Exception):pass

rar4blocks = {
    FILE_HEAD:FILE_HEADER,
}
class Rar:
    def __init__(self, fd=None):

        self.fd = None
        self.aes = None
        self.salt = None
        self.offset = None
        self.trybuf = None
        self.clearbuf = ''
        self.version = None
        self.mainhead = None


        if fd is not None:
            self.parseRarHeader(fd)

    def parseRarHeader(self, fd):

        veroff = getRarOffset(fd)
        if veroff is None:
            raise MissingRarSig()

        self.fd = fd
        self.version = veroff[0]
        self.offset = veroff[1]

        self.fd.seek(self.offset)
        self.mainhead = MAIN_HEADER()
        self.mainhead.vsParseFd( self.fd )

        if self.mainhead.HEAD_FLAGS & MHD_PASSWORD:
            self.salt = self.fd.read( SIZE_SALT30 )

    def _req_fd(self):
        if self.fd is None:
            raise NoRarFd()

    def tryFilePasswd(self, passwd):
        '''
        Check the passwd agains the next encrypted header
        ( which should be of type FILE_HEAD )
        '''
        if self.trybuf is None:
            curloc = self.fd.tell()
            self.trybuf = self.fd.read(16)
            self.fd.seek(curloc)

        iv,key = initIvKey30(passwd,self.salt)
        aes = aesInit(iv,key)
        clearbuf = aes.decrypt(self.trybuf)
        crc,ctype,cflags,csize = struct.unpack_from('<HBHH', clearbuf)
        return ctype == FILE_HEAD

    def setFilePasswd(self, passwd):
        '''
        Used to set the file-wide password for decryption.
        '''
        iv,key = initIvKey30(passwd,self.salt)
        self.aes = aesInit(iv,key)

    def read(self, size):
        if self.aes is None:
            return self.fd.read(size)

        while len(self.clearbuf) < size:
            crypted = self.fd.read(4096)
            self.clearbuf += self.aes.decrypt(crypted)

        ret = self.clearbuf[:size]
        self.clearbuf = self.clearbuf[size:]
        return ret

    #def read(self, size):
        #buf = self.fd.read(size)
        #if self.aes is not None:
            #buf = self.aes.decrypt(buf)
        #return buf

    def iterRar4Files(self):

        if self.salt is not None and self.aes is None:
            raise PasswordRequired()

        while True:
            hdr = self.read(7)
            crc,ctype,cflags,csize = struct.unpack('<HBHH', hdr)
            body = self.read(csize - 7)

            rar4 = Rar4Block()
            rar4.vsParse( hdr )

            #if self.salt is not None:
                #remain = csize % 16
                #if remain:
                    #pad = self.read( 16 - remain )
                    #logger.info('PAD %s', binascii.hexlify(pad))

            cls = rar4blocks.get(rar4.HEAD_TYPE)
            if cls is not None:
                rar4 = cls()
                rar4.vsParse(hdr+body)

            logger.info(rar4.tree())
            sys.stdin.readline()

            #if ctype == MAIN_HEAD and cflags & MHD_PASSWORD:
                #if passwd is None:
                    #raise PasswordRequired()

                #salt30 = fd.read(SIZE_SALT30)
                #iv,key = initIvKey(passwd,salt)
                #self.aes = aesInit(iv,key)
                #break

            if ctype == ENDARC_HEAD:
                break

        #self.HEAD_CRC       = v_uint16()
        #self.HEAD_TYPE      = v_uint8()
        #self.HEAD_FLAGS     = v_uint16()
        #self.HEAD_SIZE      = v_uint16()

def main():
    # TODO: Does this even work anymore?

    offset = 0
    with open(sys.argv[1], 'rb') as fd:
        testpass = sys.argv[2]

        rar = Rar()
        rar.parseRarHeader(fd)
        rar.mainhead.tree()

        #logger.info("FAIL TEST",rar.tryFilePasswd('asdf'))
        #logger.info("PASS TEST",rar.tryFilePasswd(testpass))

        rar.setFilePasswd(testpass)
        rar.iterRar4Files()
        #for x in rar.iterRar4Chunks():
            #print x
        return

        buf = fd.read(1024000)

        offset = 0

        rar4 = Rar4Block()
        offset = rar4.vsParse(buf,offset=offset)
        print(rar4.tree())

        salt = buf[offset:offset+SIZE_SALT30]
        print('SALT %s' % binascii.hexlify(salt))
        offset += SIZE_SALT30

        iv,key = initIvKey30(testpass,salt)
        #print('IV %s' % binascii.hexlify(iv))
        #print('KEY %s' % binascii.hexlify(key))
        aes = aesInit(iv,key)
        #print(binascii.hexlify(aes.decrypt(buf[offset:offset+64])))
        x = aes.decrypt(buf[offset:offset+64])

        rar4 = Rar4Block()
        rar4.vsParse(x)
        #offset = rar4.vsParse(buf,offset=offset)
        print(rar4.tree())

        #while offset < len(b):
            #r = RarBlock()
            #newoff = r.vsParse(b, offset=offset)
            #print 'CRC',r.HEAD_CRC,r.HEAD_TYPE
            #print r.tree(va=offset)

            #offset = newoff

if __name__ == '__main__':
    sys.exit(main())




import unittest

import vivisect

import vivisect.analysis.i386.golang as x86_go
import vivisect.analysis.amd64.golang as x64_go

import vivisect.tests.helpers as helpers


def loadGoBin(*path):
    '''
    We don't need full analysis on the go binaries to complete the tests
    '''
    vw = vivisect.VivWorkspace()
    path = helpers.getTestPath(*path)
    vw.loadFromFile(path)
    for ep in vw.getEntryPoints():
        vw.makeFunction(ep)
    return vw


class GolangTest(unittest.TestCase):
    '''
    Tests to verify that the GO runtime_main is discovered as a function.
    The tests succeed only if golang extended analysis is used:
    vivisect/analysis/amd64/golang.py and vivisect/analysis/i386/golang.py.
    '''
    def test_PE32(self):
        vw = loadGoBin('windows', 'i386', 'GO_hello_PE32.exe')
        x86_go.analyze(vw)
        assert vw.isFunction(0x428880)

    def test_PE32_stripped(self):
        vw = loadGoBin('windows', 'i386', 'GO_hello_PE32_stripped.exe')
        x86_go.analyze(vw)
        assert vw.isFunction(0x428880)

    def test_PE32_findcase(self):
        vw = loadGoBin('windows', 'i386', 'GO_find2case_PE32.exe')
        x86_go.analyze(vw)
        assert vw.isFunction(0x424e10)

    def test_PE64(self):
        vw = loadGoBin('windows', 'amd64', 'GO_hello_PE64.exe')
        x64_go.analyze(vw)
        assert vw.isFunction(0x42dc00)



import sys
import argparse

import cobra.cluster as c_cluster


def setup():
    ap = argparse.ArgumentParser('Cluster worker tool')
    ap.add_argument('cluster', help='Name of the cluster to attach to')
    return ap


def main(argv):
    opts = setup().parse_args(argv)
    worker = c_cluster.ClusterClient(opts.cluster, docode=True)
    worker.processWork()


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



import unittest
import envi.bytesig

class SignatureTests(unittest.TestCase):
    def test_signature_subset(self):
        signature_base = b'\x55\xe9\xd8\x01\xfe\xff\x32\x77\x89\x4f\x55'
        sigtree = envi.bytesig.SignatureTree()
        sigtree.addSignature(signature_base)

        sigtree.getSignature(b'\x55\xe9')
        sigtree.addSignature(signature_base[:7], val=signature_base[:7])
        sigtree.addSignature(signature_base[:4], val=signature_base[:4])
        sigtree.addSignature(signature_base + b'\xfe\x38', val=signature_base + b'\xfe\x38')

        self.assertTrue(sigtree.getSignature(b'\x55\xe9\xd8\x01\xfe\xff\x32\x00\x99\x36\x5f\x21\xfd') == signature_base[:7])
        self.assertTrue(sigtree.getSignature(b'\x55\xe9\xd8\x01\xfe\xff\x32') == signature_base[:7])
        self.assertTrue(sigtree.getSignature(b'\x55\xe9\xd8\x01\xfe\x00') == signature_base[:4])
        self.assertTrue(sigtree.getSignature(b'\x55') is None)



import hashlib

import envi
import envi.archs.amd64 as e_amd64

from vivisect.symboliks.common import *
import vivisect.symboliks.archs.i386 as vsym_i386
import vivisect.symboliks.analysis as vsym_analysis
import vivisect.symboliks.callconv as vsym_callconv
from envi.archs.amd64.vmcslookup import VMCS_NAMES

class VMCS_Field(Var):
    def __init__(self, offset, width):
        SymbolikBase.__init__(self)
        self.offset = offset
        self.width = width

    def __repr__(self):
        return 'VMCS_Field(%s,width=%d)' % (self.offset, self.width)

    def __str__(self):
        if self._strval:
            return self._strval

        if not self.offset.isDiscrete():
            return 'VMCS::%s' % self.offset

        self._strval = 'VMCS_%s' % VMCS_NAMES.get(self.offset.solve())
        return self._strval

    def _solve(self, emu=None):
        name = 'vmcs%s' % self.offset

        if emu is not None:
            name += emu.getRandomSeed()

        return int(hashlib.md5(name).hexdigest()[:self.width*2], 16)

    def update(self, emu):
        offset = self.offset.update(emu=emu)
        return VMCS_Field(offset, width=self.width)

    def _reduce(self, emu=None):
        self.offset._reduce(emu=emu)
        return self

    def getWidth(self):
        return self.width

    def isDiscrete(self, emu=None):
        return False


class Amd64SymbolikTranslator(vsym_i386.IntelSymbolikTranslator):
    __arch__ = e_amd64.Amd64Module
    __ip__ = 'rip'  # we could use regctx.getRegisterName if we want.
    __sp__ = 'rsp'  # we could use regctx.getRegisterName if we want.
    __bp__ = 'rbp'  # we could use regctx.getRegisterName if we want.
    __srcp__ = 'rsi'
    __destp__ = 'rdi'

    def setRegObj(self, regidx, obj):
        ridx = regidx & 0xffff
        rname = self._reg_ctx.getRegisterName(ridx)
        rbitwidth = self._reg_ctx.getRegisterWidth(ridx)
        val = Var(rname, rbitwidth >> 3)

        # Translate to native if needed...
        if ridx != regidx:
            # 64 bit mode setting to 32bit regs, 0-extends to 64 bits
            if regidx == ridx | e_amd64.RMETA_LOW32:
                val = Var(rname, 8)
            else:
                # we cannot call _xlateToNativReg since we'd pass in a symbolik
                # object that would trigger an or operation; the code in envi
                # obviously is NOT symboliks aware (2nd op in | operation is NOT
                # a symbolik); so we do it manually here since we are symbolik
                # aware.
                # obj = self._reg_ctx._xlateToNativeReg(regidx, obj)
                basemask = (2**rbitwidth) - 1
                rreg, lshift, mask = self._reg_ctx.getMetaRegInfo(regidx)
                # cut hole in mask
                finalmask = basemask ^ (mask << lshift)
                if lshift != 0:
                    obj <<= Const(lshift, rbitwidth >> 3)

                obj = obj | (val & Const(finalmask, rbitwidth >> 3))

        self.effSetVariable(rname, obj)

    def getOperAddrObj(self, op, idx):
        oper = op.opers[idx]
        if isinstance(oper, e_amd64.Amd64RipRelOper):
            return Const(op.va + len(op) + oper.imm, 8)

        return vsym_i386.IntelSymbolikTranslator.getOperAddrObj(self, op, idx)

    def getOperObj(self, op, idx):
        oper = op.opers[idx]
        if isinstance(oper, e_amd64.Amd64RipRelOper):
            return Mem(Const(op.va + len(op) + oper.imm, 8), Const(oper.tsize, 8))

        return vsym_i386.IntelSymbolikTranslator.getOperObj(self, op, idx)

    # TODO: support callf and all that nonsense

    def i_pextrd_q(self, op):
        self.i_pextrb(op, width=op.opers[0].tsize)

    def i_movsxd(self, op):
        dsize = op.opers[0].tsize
        ssize = op.opers[1].tsize
        v2 = o_sextend(self.getOperObj(op, 1), Const(dsize, self._psize))
        self.setOperObj(op, 0, v2)

    def _div(self, op, isInvalid=None):
        oper = op.opers[0]
        denom = self.getOperObj(op, 0)
        if denom == 0:
            # TODO: make effect
            raise envi.DivideByZero('AMD64 Divide by zero')

        if isInvalid is None:
            limit = (2 ** (oper.tsize * 8)) - 1
            isInvalid = lambda val: val > limit

        if oper.tsize == 8:
            rax = Var('rax', self._psize)
            rdx = Var('rdx', self._psize)
            num = (rdx << Const(64, self._psize)) | rax
            temp = num / denom
            if temp.isDiscrete() and isInvalid(temp):
                # TODO: make effect
                raise envi.DivideError('AMD64 i_div #DE')

            self.effSetVariable('rax', temp)
            self.effSetVariable('rdx', num % denom)

            return

        return vsym_i386.IntelSymbolikTranslator._div(self, op, isInvalid=isInvalid)

    def i_div(self, op):
        return self._div(op)

    def i_jecxz(self, op):
        return vsym_i386.IntelSymbolikTranslator.i_jecxz(self, op)

    def i_jrcxz(self, op):
        return self._cond_jmp(op, eq(Var('rcx', self._psize), Const(0, self._psize)))

    def i_pushfd(self, op):
        sp = self.getRegObj(self._reg_ctx._rctx_spindex)
        sr = self.getRegObj(self._reg_ctx._rctx_srindex)
        self.effSetVariable(self.__sp__, sp - Const(8, self._psize))
        self.effWriteMemory(Var(self.__sp__, self._psize), Const(8, self._psize), sr)

    def i_vmread(self, op):
        vmcsoff = self.getOperObj(op, 1)
        self.setOperObj(op, 0, LookupVar("VMCS", vmcsoff, VMCS_NAMES, vmcsoff.getWidth()))

    # FIXME CATASTROPHIC THIS CONTAINS BRANCHING LOGIC STATE!
    # FOR NOW WE JUST DO IT WITHOUT ANY CONDITIONAL (see i386.i_cmpxchg)
    i_cmova = vsym_i386.IntelSymbolikTranslator.i_mov
    i_cmovae = vsym_i386.IntelSymbolikTranslator.i_mov
    i_cmovnb = i_cmovae

    i_cmovb = vsym_i386.IntelSymbolikTranslator.i_mov
    i_cmovbe = vsym_i386.IntelSymbolikTranslator.i_mov
    i_cmovna = i_cmovbe
    i_cmovnae = i_cmovb

    i_cmovc = vsym_i386.IntelSymbolikTranslator.i_mov
    i_cmovnc = vsym_i386.IntelSymbolikTranslator.i_mov

    i_cmove = vsym_i386.IntelSymbolikTranslator.i_mov
    i_cmovne = vsym_i386.IntelSymbolikTranslator.i_mov

    i_cmovg = vsym_i386.IntelSymbolikTranslator.i_mov
    i_cmovge = vsym_i386.IntelSymbolikTranslator.i_mov
    i_cmovnle = i_cmovg
    i_cmovnl = i_cmovge

    i_cmovl = vsym_i386.IntelSymbolikTranslator.i_mov
    i_cmovle = vsym_i386.IntelSymbolikTranslator.i_mov
    i_cmovnge = i_cmovl
    i_cmovng = i_cmovle

    i_cmovo = vsym_i386.IntelSymbolikTranslator.i_mov
    i_cmovno = vsym_i386.IntelSymbolikTranslator.i_mov

    i_cmovp = vsym_i386.IntelSymbolikTranslator.i_mov
    i_cmovpe = vsym_i386.IntelSymbolikTranslator.i_mov
    i_cmovnp = vsym_i386.IntelSymbolikTranslator.i_mov
    i_cmovnpe = vsym_i386.IntelSymbolikTranslator.i_mov

    i_cmovs = vsym_i386.IntelSymbolikTranslator.i_mov
    i_cmovns = vsym_i386.IntelSymbolikTranslator.i_mov

    i_cmovz = i_cmove
    i_cmovnz = i_cmovne

    def i_movsq(self, op):
        return self._movs(op, width=8)

    def i_stosq(self, op):
        return self._stos(op, width=8)

    def i_cmpsq(self, op):
        return self._cmps(op, width=8)

class Amd64ArgDefSymEmu(vsym_callconv.ArgDefSymEmu):
    __xlator__ = Amd64SymbolikTranslator

class MSx64CallSym(vsym_callconv.SymbolikCallingConvention, e_amd64.MSx64Call):
    __argdefemu__ = Amd64ArgDefSymEmu

class SysVAmd64CallSym(vsym_callconv.SymbolikCallingConvention, e_amd64.SysVAmd64Call):
    __argdefemu__ = Amd64ArgDefSymEmu

class Amd64SymFuncEmu(vsym_analysis.SymbolikFunctionEmulator):
    __width__ = 8

    def __init__(self, vw, *args, initial_sp=0xbfbff000, xlator=None):
        vsym_analysis.SymbolikFunctionEmulator.__init__(self, vw, *args, xlator=xlator)
        self.setStackBase(0xbfbff000, 16384)
        self.addCallingConvention('sysvamd64call', SysVAmd64CallSym(xlator))
        self.addCallingConvention('msx64call', MSx64CallSym(xlator))

    def getStackCounter(self):
        return self.getSymVariable('rsp')

    def setStackCounter(self, symobj):
        self.setSymVariable('rsp', symobj)

class Amd64SymbolikAnalysisContext(vsym_analysis.SymbolikAnalysisContext):
    __xlator__ = Amd64SymbolikTranslator
    __emu__ = Amd64SymFuncEmu
